from __future__ import unicode_literals

from docker_base import BuildContext
from docker_base import DefaultNaming
from docker_base import UserMixin
from docker_base import HomeMountsMixin
from docker_base import ProjectMountMixin


class ContainerBuildContext(ProjectMountMixin, UserMixin, HomeMountsMixin,
                            DefaultNaming, BuildContext):
    pass



import os

from os.path import join, expanduser

from ds.presets.docker_base.mixins import UserMixin
from ds.presets.docker_base.mixins import HomeMountsMixin


def test_user():
    assert UserMixin().get_run_options().get('user') == os.getuid()

    class Context(UserMixin):
        container_user = 999

    assert Context().get_run_options().get('user') == Context.container_user


def test_home_mounts():
    class Context(HomeMountsMixin):
        container_home = 'a', 'b',
        home_mounts = [
            '.',
            '..',
        ]

    context = Context()

    mounts = {
        item['Target']: item['Source']
        for item in context.get_run_options().get('mounts')
    }

    home = expanduser('~')

    for dest in context.container_home:
        for src in context.home_mounts:
            full_dst = join(dest, src)
            assert full_dst in mounts

            full_src = join(home, src)
            assert mounts[full_dst] == full_src



from ds.environment import get_environment


context = 'container_simple'


def test_switch_context(shell):
    shell.call_ds('switch-context', context)
    assert context == get_environment().get('context')


def test_bad_cmd(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_docker_ds('start', 'none')
    assert result.code == 1


def test_echo(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_docker_ds('start', '/bin/bash', '-c', 'echo ok')
    assert result.code == 0
    assert result.stdout.strip() == 'ok'



import os
from os.path import exists
from os.path import expanduser
from os.path import join

from docker.types import Mount

from .contexts import DockerContext


class UserMixin(DockerContext):
    @property
    def container_user(self):
        return os.getuid()

    def get_run_options(self, **options):
        return super(UserMixin, self).\
            get_run_options(user=self.container_user, **options)


class HomeMountsMixin(DockerContext):
    container_home = '/',

    home_mounts = [
        '.bashrc',
        '.inputrc',
        '.config/bash',
        '.psqlrc',
        '.liquidpromptrc',
    ]

    def get_mounts(self):
        additional = []
        for src in self.home_mounts:
            full_src = expanduser(join('~', src))
            if not exists(full_src):
                continue
            for container_home in self.container_home:
                full_dest = join(container_home, src)
                mount = Mount(target=full_dest,
                              source=full_src,
                              type='bind',
                              read_only=True)
                additional.append(mount)
        return super(HomeMountsMixin, self).get_mounts() + additional


class ProjectMountMixin(DockerContext):
    working_dir = '/app/'

    @property
    def project_mount_path(self):
        return self.project_root

    @property
    def project_mount_readonly(self):
        return False

    def get_run_options(self, **options):
        return super(ProjectMountMixin, self). \
            get_run_options(working_dir=self.working_dir, **options)

    def get_mounts(self):
        mount = Mount(target=self.working_dir,
                      source=self.project_mount_path,
                      type='bind',
                      read_only=self.project_mount_readonly)
        return super(ProjectMountMixin, self).get_mounts() + [mount]



from .contexts import DockerContextMixin
from .contexts import ExternalContext
from .contexts import BuildContext
from .contexts import PullContext
from .naming import ContainerNaming
from .naming import ImageNaming
from .naming import DefaultNaming
from .mixins import UserMixin
from .mixins import HomeMountsMixin
from .mixins import ProjectMountMixin



from __future__ import unicode_literals
import sys

try:
    import docker
    import docker.errors
    from docker.types import Mount
except ImportError:
    print('Install docker-py with `pip install docker`')
    sys.exit(1)

from ds import context
from ds.summary import TableSummary
from ds.utils import drop_empty
from ds.presets.docker_base import commands
from ds.presets.docker_base import naming


class DockerContextMixin(context.Context):
    def __init__(self):
        super(DockerContextMixin, self).__init__()
        self._client = None

    @property
    def client(self):
        if self._client is None:
            self._client = docker.from_env()
        return self._client


class DockerContext(naming.ContainerNaming, DockerContextMixin,
                    context.Context):
    """"""

    stop_before_start = True
    remove_before_start = True

    detach_keys = 'ctrl-c'

    logs_tail = 100

    def __init__(self):
        super(DockerContext, self).__init__()
        self._container = None

    def get_commands(self):
        return super(DockerContext, self).get_commands() + drop_empty(
            commands.ShowRunOptions if self.has_image_name else None,
            commands.Create if self.has_image_name else None,
            commands.Start if self.has_image_name else None,
            commands.Stop,
            commands.Recreate if self.has_image_name else None,
            commands.Restart if self.has_image_name else None,
            commands.Kill,
            commands.Rm,
            commands.Logs,
            commands.Attach,
            commands.Exec,
            commands.Shell,
            commands.RootShell,
        )

    @property
    def container(self):
        if not self.container_name:
            return
        try:
            return self.client.containers.get(self.container_name)
        except docker.errors.NotFound:
            pass

    def get_run_options(self, **options):
        """
        https://docker-py.readthedocs.io/en/stable/containers.html#docker.models.containers.ContainerCollection.run
        """
        result = dict(
            detach=False,
            auto_remove=True,
            stdin_open=True,
            tty=True,
            mounts=self.get_mounts(),
        )
        result.update(options)
        return result

    def get_mounts(self):
        return []

    def get_additional_summary(self):
        container = self.container
        cells = [
            ['Name', self.container_name or '-'],
            ['Image', self.image_name or '-'],
            ['Status', container.status if container else '-'],
            ['ID', container.short_id if container else '-'],
        ]
        return super(DockerContext, self).get_additional_summary() + [
            TableSummary('Container', cells),
        ]


class ExternalContext(DockerContext):
    pass


class BuildContext(naming.ImageNaming, DockerContext):
    def get_commands(self):
        return super(BuildContext, self).get_commands() + [
            commands.Build,
        ]


class PullContext(naming.ImageNaming, DockerContext):
    default_image = None
    default_tag = 'latest'

    def check(self):
        assert self.default_image, 'Default image is not set'
        super(PullContext, self).check()

    @property
    def image_name(self):
        if not self.default_image:
            return
        return ':'.join([self.default_image, self.default_tag])

    def get_commands(self):
        return super(PullContext, self).get_commands() + [
            commands.Pull,
        ]
